@@ -581,8 +581,20 @@ class BeamMonitor(Element):
 
 
 class DipoleEdge(Element):
+
+    @classmethod
+    def from_dict(cls, dct):
+        dct = dct.copy()
+        for kk in list(dct.keys()):
+            if kk == "h" or kk == "_h":
+                dct["k"] = dct[kk]
+                continue
+            if kk.startswith("_"):
+                dct[kk[1:]] = dct[kk]
+        return super(DipoleEdge, cls).from_dict(dct)
+
     _description = [
-        ("h", "1/m", "Curvature", 0),
+        ("k", "1/m", "Curvature", 0),
         ("e1", "rad", "Face angle", 0),
         ("hgap", "m", "Equivalent gap", 0),
         ("fint", "", "Fringe integral", 0),
@@ -592,9 +604,9 @@ class DipoleEdge(Element):
         tan = p._m.tan
         sin = p._m.sin
         cos = p._m.cos
-        corr = 2 * self.h * self.hgap * self.fint
-        r21 = self.h * tan(self.e1)
-        r43 = -self.h * tan(
+        corr = 2 * self.k * self.hgap * self.fint
+        r21 = self.k * tan(self.e1)
+        r43 = -self.k * tan(
             self.e1 - corr / cos(self.e1) * (1 + sin(self.e1) ** 2)
         )
         p.px += r21 * p.x
@@ -605,25 +617,29 @@ class LinearTransferMatrix(Element):
         ("alpha_x_0","","",0.0),
         ("beta_x_0","","",0.0),
         ("disp_x_0","","",0.0),
+        ("disp_px_0","","",0.0),
         ("alpha_x_1","","",0.0),
         ("beta_x_1","","",0.0),
         ("disp_x_1","","",0.0),
+        ("disp_px_1","","",0.0),
         ("alpha_y_0","","",0.0),
         ("beta_y_0","","",0.0),
         ("disp_y_0","","",0.0),
+        ("disp_py_0","","",0.0),
         ("alpha_y_1","","",0.0),
         ("beta_y_1","","",0.0),
         ("disp_y_1","","",0.0),
+        ("disp_py_1","","",0.0),
         ("Q_x","","",0.0),
         ("Q_y","","",0.0),
         ("beta_s","","",0.0),
         ("Q_s","","",0.0),
         ("chroma_x","","",0.0),
         ("chroma_y","","",0.0),
-        ("detx_x","","",0.0),
-        ("detx_y","","",0.0),
-        ("dety_y","","",0.0),
-        ("dety_x","","",0.0),
+        ("det_xx","","",0.0),
+        ("det_xy","","",0.0),
+        ("det_yy","","",0.0),
+        ("det_yx","","",0.0),
         ("energy_ref_increment","","",0.0),
         ("energy_increment","","",0.0),
         ("x_ref_0","","",0.0),
@@ -659,10 +675,16 @@ class LinearTransferMatrix(Element):
         #Transverse linear uncoupled matrix
 
         # removing dispersion and close orbit
+        old_x=p.x
+        old_px=p.px
+        old_y=p.y
+        old_py=p.py
+
         p.x -= self.disp_x_0 * p.delta + self.x_ref_0
-        p.px -= self.px_ref_0
+        p.px -= self.disp_px_0 * p.delta + self.px_ref_0
         p.y -= self.disp_y_0 * p.delta + self.y_ref_0
-        p.py -= self.py_ref_0
+        p.py -= self.disp_py_0 * p.delta + self.py_ref_0
+        # p.zeta += (self.disp_px_0*old_x - self.disp_x_0*old_px + self.disp_py_0*old_y - self.disp_y_0*old_py)/p.rvv
 
         J_x = 0.5 * (
                 (1.0 + self.alpha_x_0*self.alpha_x_0)/self.beta_x_0 * p.x*p.x
@@ -672,10 +694,10 @@ class LinearTransferMatrix(Element):
                 (1.0 + self.alpha_y_0*self.alpha_y_0)/self.beta_y_0 * p.y*p.y
                 + 2*self.alpha_y_0 * p.y*p.py
                 + self.beta_y_0 * p.py*p.py)
-        phase = 2*np.pi*(self.Q_x+self.chroma_x*p.delta+self.detx_x*J_x+self.detx_y*J_y)
+        phase = 2*np.pi*(self.Q_x+self.chroma_x*p.delta+self.det_xx*J_x+self.det_xy*J_y)
         cos_x = cos(phase)
         sin_x = sin(phase)
-        phase = 2*np.pi*(self.Q_y+self.chroma_y*p.delta+self.dety_y*J_y+self.dety_x*J_x)
+        phase = 2*np.pi*(self.Q_y+self.chroma_y*p.delta+self.det_yy*J_y+self.det_yx*J_x)
         cos_y = cos(phase)
         sin_y = sin(phase)
 
@@ -695,8 +717,8 @@ class LinearTransferMatrix(Element):
         p.x,p.px = M00_x*p.x + M01_x*p.px, M10_x*p.x + M11_x*p.px
         p.y,p.py = M00_y*p.y + M01_y*p.py, M10_y*p.y + M11_y*p.py
 
-        pzeta_new = -sin_s*p.zeta/self.beta_s+cos_s*p.pzeta
-        zeta_new = cos_s*p.zeta+self.beta_s*sin_s*p.pzeta
+        pzeta_new = sin_s*p.zeta/self.beta_s+cos_s*p.pzeta
+        zeta_new = cos_s*p.zeta-self.beta_s*sin_s*p.pzeta
 
         p.zeta = zeta_new
         p.pzeta = pzeta_new
@@ -709,6 +731,8 @@ class LinearTransferMatrix(Element):
         #  both the position and the momentum are scaled,
         #  rather than only the momentum.
         if self.energy_ref_increment != 0:
+            old_px = p.px.copy()
+            old_py = p.py.copy()
             new_energy0 = p.mass0*p.gamma0 + self.energy_ref_increment
             new_p0c = sqrt(new_energy0*new_energy0-p.mass0*p.mass0)
             new_beta0 = new_p0c / new_energy0
@@ -718,9 +742,9 @@ class LinearTransferMatrix(Element):
             p.p0c = new_p0c
 
             p.x *= geo_emit_factor
-            p.px *= geo_emit_factor
+            p.px = old_px * geo_emit_factor
             p.y *= geo_emit_factor
-            p.py *= geo_emit_factor
+            p.py = old_py * geo_emit_factor
 
         if self.damping_rate_x < 0.0 or self.damping_rate_y < 0.0 or self.damping_rate_s < 0.0:
             raise ValueError("Damping rates cannot be negative")
@@ -755,10 +779,17 @@ class LinearTransferMatrix(Element):
             p.delta += self.gauss_noise_ampl_s*np.random.randn(len(p.delta))
 
         # re-adding dispersion and closed orbit
+        old_x=p.x
+        old_px=p.px
+        old_y=p.y
+        old_py=p.py
+
         p.x += self.disp_x_1 * p.delta + self.x_ref_1
-        p.px += self.px_ref_1
+        p.px += self.disp_px_1 * p.delta + self.px_ref_1 
         p.y += self.disp_y_1 * p.delta + self.y_ref_1
-        p.py += self.py_ref_1
+        p.py += self.disp_py_1 * p.delta + self.py_ref_1
+        # p.zeta-= (self.disp_px_1*old_x - self.disp_x_1*old_px + self.disp_py_1*old_y - self.disp_y_1*old_py)/p.rvv
+
 
 class FirstOrderTaylorMap(Element):
     _description = [
