@@ -24,7 +24,7 @@ void LimitEllipse_track_local_particle(LimitEllipseData el, LocalParticle* part0
 
 	// I assume that if I am in the function is because
     	if (!is_alive){
-           LocalParticle_set_state(part, 0);
+           LocalParticle_set_state(part, XT_LOST_ON_APERTURE);
 	}
 
     //end_per_particle_block
