@@ -31,7 +31,7 @@ void LimitRectEllipse_track_local_particle(LimitRectEllipseData el, LocalParticl
 
 	// I assume that if I am in the function is because
     	if (!is_alive){
-           LocalParticle_set_state(part, 0);
+           LocalParticle_set_state(part, XT_LOST_ON_APERTURE);
 	}
 
     //end_per_particle_block
