@@ -10,7 +10,11 @@
 void ReferenceEnergyIncrease_track_local_particle(ReferenceEnergyIncreaseData el,
 		                                  LocalParticle* part0){
 
-    double const Delta_p0c = ReferenceEnergyIncreaseData_get_Delta_p0c(el);
+    double Delta_p0c = ReferenceEnergyIncreaseData_get_Delta_p0c(el);
+
+    #ifdef XSUITE_BACKTRACK
+        Delta_p0c = -Delta_p0c;
+    #endif
 
     //start_per_particle_block (part0->part)
 	LocalParticle_update_p0c(part,
