@@ -27,20 +27,21 @@ Loader.add_<name>(mad_elem,line,buffer) to add a new element to line
 
 if the want to control how the xobject is created
 """
-
-from typing import List
+from typing import List, Union
 
 import numpy as np
 
-import xtrack, xobjects
-
+import xobjects
+import xtrack
 from .general import _print
-
+from .progress_indicator import progress
 
 # Generic functions
 
 clight = 299792458
 
+DEFAULT_BEND_N_MULT_KICKS = 5
+
 
 def iterable(obj):
     return hasattr(obj, "__iter__")
@@ -55,9 +56,6 @@ def rad2deg(rad):
     return rad * 180 / np.pi
 
 
-
-
-
 def get_value(x):
     if is_expr(x):
         return x._get_value()
@@ -129,13 +127,21 @@ def trim_trailing_zeros(lst):
 def is_expr(x):
     return hasattr(x, "_get_value")
 
-def not_zero(x):
+
+def nonzero_or_expr(x):
     if is_expr(x):
         return True
     else:
         return x != 0
 
 
+def value_if_expr(x):
+    if is_expr(x):
+        return x._value
+    else:
+        return x
+
+
 def eval_list(par, madeval):
     if madeval is None:
         return par.value
@@ -168,8 +174,11 @@ class PhaseErrors:
 
 
 class MadElem:
-    def __init__(self, name, elem, sequence, madeval=None):
-        self.name = name
+    def __init__(self, name, elem, sequence, madeval=None, name_prefix=None):
+        if name_prefix is None:
+            self.name = name
+        else:
+            self.name = name_prefix + name
         self.elem = elem
         self.sequence = sequence
         self.madeval = madeval
@@ -189,6 +198,16 @@ class MadElem:
     #    if hasattr(elem, "field_errors") and elem.field_errors is not None:
     #        return FieldErrors(elem.field_errors)
 
+    def get_type_hierarchy(self, cpymad_elem=None):
+        if cpymad_elem is None:
+            cpymad_elem = self.elem
+
+        if cpymad_elem.name == cpymad_elem.parent.name:
+            return [cpymad_elem.name]
+
+        parent_types = self.get_type_hierarchy(cpymad_elem.parent)
+        return [cpymad_elem.name] + parent_types
+
     @property
     def phase_errors(self):
         elem = self.elem
@@ -289,22 +308,38 @@ class ElementBuilder:
         return "Element(%s, %s, %s)" % (self.name, self.type, self.attrs)
 
     def __setattr__(self, k, v):
-        if hasattr(self, "attrs"):
+        if hasattr(self, "attrs") and k not in ('name', 'type', 'attrs'):
             self.attrs[k] = v
         else:
             super().__setattr__(k, v)
 
     def add_to_line(self, line, buffer):
+        if self.type is xtrack.Drift:
+            self.attrs.pop("rot_s_rad", None)
+            self.attrs.pop("shift_x", None)
+            self.attrs.pop("shift_y", None)
+        name_associated_aperture = self.attrs.pop("name_associated_aperture", None)
         xtel = self.type(**self.attrs, _buffer=buffer)
+        if name_associated_aperture:
+            xtel.name_associated_aperture = name_associated_aperture
         name = generate_repeated_name(line, self.name)
         line.append_element(xtel, name)
 
 
 class ElementBuilderWithExpr(ElementBuilder):
     def add_to_line(self, line, buffer):
+
+        if self.type is xtrack.Drift:
+            self.attrs.pop("rot_s_rad", None)
+            self.attrs.pop("shift_x", None)
+            self.attrs.pop("shift_y", None)
+
         attr_values = {k: get_value(v) for k, v in self.attrs.items()}
+        name_associated_aperture = attr_values.pop("name_associated_aperture", None)
         xtel = self.type(**attr_values, _buffer=buffer)
         name = generate_repeated_name(line, self.name)
+        if name_associated_aperture:
+            xtel.name_associated_aperture = name_associated_aperture
         line.append_element(xtel, name)
         elref = line.element_refs[name]
         for k, p in self.attrs.items():
@@ -315,7 +350,7 @@ class ElementBuilderWithExpr(ElementBuilder):
 class Aperture:
     def __init__(self, mad_el, enable_errors, loader):
         self.mad_el = mad_el
-        self.aper_tilt = rad2deg(mad_el.aper_tilt)
+        self.aper_tilt = mad_el.aper_tilt
         self.aper_offset = mad_el.aper_offset
         self.name = self.mad_el.name
         self.dx = self.aper_offset[0]
@@ -331,71 +366,39 @@ class Aperture:
         self.classes = loader.classes
         self.Builder = loader.Builder
 
-    def entry(self):
-        out = []
-        if self.aper_tilt:
-            out.append(
-                self.Builder(
-                    self.name + "_aper_tilt_entry",
-                    self.classes.SRotation,
-                    angle=self.aper_tilt,
-                )
-            )
-        if self.dx or self.dy:
-            out.append(
-                self.Builder(
-                    self.name + "_aper_offset_entry",
-                    self.classes.XYShift,
-                    dx=self.dx,
-                    dy=self.dy,
-                )
-            )
-        return out
-
-    def exit(self):
-        out = []
-        if self.dx or self.dy:
-            out.append(
-                self.Builder(
-                    self.name + "_aper_offset_exit",
-                    self.classes.XYShift,
-                    dx=-self.dx,
-                    dy=-self.dy,
-                )
-            )
-        if not_zero(self.aper_tilt):
-            out.append(
-                self.Builder(
-                    self.name + "_aper_tilt_exit",
-                    self.classes.SRotation,
-                    angle=-self.aper_tilt,
-                )
-            )
-        return out
-
     def aperture(self):
         if len(self.mad_el.aper_vx) > 2:
-            return [
-                self.Builder(
+            builder = self.Builder(
                     self.name + "_aper",
                     self.classes.LimitPolygon,
                     x_vertices=self.mad_el.aper_vx,
                     y_vertices=self.mad_el.aper_vy,
                 )
-            ]
+            if self.dx or self.dy or self.aper_tilt:
+                builder.shift_x = self.dx
+                builder.shift_y = self.dy
+                builder.rot_s_rad = self.aper_tilt
+            return [builder]
         else:
             conveter = getattr(self.loader, "convert_" + self.apertype, None)
             if conveter is None:
                 raise ValueError(f"Aperture type `{self.apertype}` not supported")
-            return conveter(self.mad_el)
+            out = conveter(self.mad_el)
+            assert len(out) == 1
+            if self.dx or self.dy or self.aper_tilt:
+                out[0].shift_x = self.dx
+                out[0].shift_y = self.dy
+                out[0].rot_s_rad = self.aper_tilt
+            return out
 
 
 class Alignment:
-    def __init__(self, mad_el, enable_errors, classes, Builder):
+    def __init__(self, mad_el, enable_errors, classes, Builder, bv, custom_tilt=None):
         self.mad_el = mad_el
-        self.tilt = mad_el.get("tilt", 0)  # some elements do not have tilt
-        if self.tilt:
-            self.tilt = rad2deg(self.tilt)
+        self.bv = bv
+        self.tilt = bv * mad_el.get("tilt", 0)  # some elements do not have tilt
+        if custom_tilt is not None:
+            self.tilt += custom_tilt
         self.name = mad_el.name
         self.dx = 0
         self.dy = 0
@@ -404,70 +407,32 @@ class Alignment:
             and hasattr(mad_el, "align_errors")
             and mad_el.align_errors is not None
         ):
+            if bv != 1:
+                raise NotImplementedError("Alignment errors not supported for bv=-1")
             self.align_errors = mad_el.align_errors
             self.dx = self.align_errors.dx
             self.dy = self.align_errors.dy
-            self.tilt += rad2deg(self.align_errors.dpsi)
+            self.tilt += self.align_errors.dpsi
         self.classes = classes
         self.Builder = Builder
 
-    def entry(self):
-        out = []
-        if self.tilt:
-            out.append(
-                self.Builder(
-                    self.name + "_tilt_entry",
-                    self.classes.SRotation,
-                    angle=self.tilt,
-                )
-            )
-        if self.dx or self.dy:
-            out.append(
-                self.Builder(
-                    self.name + "_offset_entry",
-                    self.classes.XYShift,
-                    dx=self.dx,
-                    dy=self.dy,
-                )
-            )
-        return out
-
-    def exit(self):
-        out = []
-        if self.dx or self.dy:
-            out.append(
-                self.Builder(
-                    self.name + "_offset_exit",
-                    self.classes.XYShift,
-                    dx=-self.dx,
-                    dy=-self.dy,
-                )
-            )
-        if self.tilt:
-            out.append(
-                self.Builder(
-                    self.name + "_tilt_exit",
-                    self.classes.SRotation,
-                    angle=-self.tilt,
-                )
-            )
-        return out
-
-
 class Dummy:
     type = "None"
 
+def _default_factory():
+    return 0.
 
 class MadLoader:
     @staticmethod
     def init_line_expressions(line, mad, replace_in_expr):  # to be added to Line....
         """Enable expressions"""
-        line._init_var_management()
+        if line._var_management is None:
+            line._init_var_management()
 
         from xdeps.madxutils import MadxEval
 
         _var_values = line._var_management["data"]["var_values"]
-        _var_values.default_factory = lambda: 0
+        _var_values.default_factory = _default_factory
         for name, par in mad.globals.cmdpar.items():
             if replace_in_expr is not None:
                 for k, v in replace_in_expr.items():
@@ -478,7 +443,7 @@ class MadLoader:
         _fref = line._var_management["fref"]
         _lref = line._var_management["lref"]
 
-        madeval_no_repl = MadxEval(_vref, _fref, None).eval
+        madeval_no_repl = MadxEval(_vref, _fref, mad.elements).eval
 
         if replace_in_expr is not None:
             def madeval(expr):
@@ -501,7 +466,9 @@ class MadLoader:
         self,
         sequence,
         enable_expressions=False,
-        enable_errors=False,
+        enable_errors=None,
+        enable_field_errors=None,
+        enable_align_errors=None,
         enable_apertures=False,
         skip_markers=False,
         merge_drifts=False,
@@ -510,16 +477,43 @@ class MadLoader:
         ignore_madtypes=(),
         expressions_for_element_types=None,
         classes=xtrack,
-        replace_in_expr=None
+        replace_in_expr=None,
+        allow_thick=False,
+        name_prefix=None,
+        enable_layout_data=False,
     ):
 
+
+        if enable_errors is not None:
+            if enable_field_errors is None:
+                enable_field_errors = enable_errors
+            if enable_align_errors is None:
+                enable_align_errors = enable_errors
+
+        if enable_field_errors is None:
+            enable_field_errors = False
+        if enable_align_errors is None:
+            enable_align_errors = False
+
+        if allow_thick is None:
+            if enable_field_errors:
+                allow_thick = False
+            else:
+                allow_thick = True
+
+        if allow_thick and enable_field_errors:
+            raise NotImplementedError(
+                "Field errors are not yet supported for thick elements"
+            )
+
         if expressions_for_element_types is not None:
             assert enable_expressions, ("Expressions must be enabled if "
                                 "`expressions_for_element_types` is not None")
 
         self.sequence = sequence
         self.enable_expressions = enable_expressions
-        self.enable_errors = enable_errors
+        self.enable_field_errors = enable_field_errors
+        self.enable_align_errors = enable_align_errors
         self.error_table = error_table
         self.skip_markers = skip_markers
         self.merge_drifts = merge_drifts
@@ -530,12 +524,26 @@ class MadLoader:
         self.replace_in_expr = replace_in_expr
         self._drift = self.classes.Drift
         self.ignore_madtypes = ignore_madtypes
+        self.name_prefix = name_prefix
+        self.enable_layout_data = enable_layout_data
+
+        self.allow_thick = allow_thick
+        self.bv = 1
 
     def iter_elements(self, madeval=None):
         """Yield element data for each known element"""
+        if len(self.sequence.expanded_elements)==0:
+            raise ValueError(f"{self.sequence} has no elements, please do {self.sequence}.use()")
         last_element = Dummy
-        for el in self.sequence.expanded_elements:
-            madelem = MadElem(el.name, el, self.sequence, madeval)
+        if self.bv == -1:
+            expanded_elements = list(self.sequence.expanded_elements)[::-1]
+        elif self.bv == 1:
+            expanded_elements = self.sequence.expanded_elements
+        else:
+            raise ValueError(f"bv should be 1 or -1, not {self.bv}")
+        for el in expanded_elements:
+            madelem = MadElem(el.name, el, self.sequence, madeval,
+                              name_prefix=self.name_prefix)
             if self.skip_markers and madelem.is_empty_marker():
                 pass
             elif (
@@ -563,12 +571,14 @@ class MadLoader:
 
     def make_line(self, buffer=None):
         """Create a new line in buffer"""
+
         mad = self.sequence._madx
 
         if buffer is None:
             buffer = xobjects.context_default.new_buffer()
 
         line = self.classes.Line()
+        self.line = line
 
         if self.enable_expressions:
             madeval = MadLoader.init_line_expressions(line, mad,
@@ -578,9 +588,20 @@ class MadLoader:
             madeval = None
             self.Builder = ElementBuilder
 
-        nelem = len(self.sequence.expanded_elements)
+        bv = self.sequence.beam.bv
+        assert bv==1 or bv==-1, f"bv should be 1 or -1, not {bv}"
+        self.bv = bv
 
-        for ii, el in enumerate(self.iter_elements(madeval=madeval)):
+        # Avoid progress bar if there are few elements
+        if len(self.sequence.expanded_elements) > 10:
+            _prog = progress(
+                self.iter_elements(madeval=madeval),
+                desc=f'Converting sequence "{self.sequence.name}"',
+                total=len(self.sequence.expanded_elements))
+        else:
+            _prog = self.iter_elements(madeval=madeval)
+
+        for ii, el in enumerate(_prog):
             # for each mad element create xtract elements in a buffer and add to a line
             converter = getattr(self, "convert_" + el.type, None)
             adder = getattr(self, "add_" + el.type, None)
@@ -601,48 +622,257 @@ class MadLoader:
                     f'Element {el.type} not supported,\nimplement "add_{el.type}"'
                     f" or convert_{el.type} in function in MadLoader"
                 )
-            if ii % 100 == 0:
-                _print(
-                    f'Converting sequence "{self.sequence.name}":'
-                    f' {round(ii/nelem*100):2d}%     ',
-                    end="\r",
-                    flush=True,
-                )
-        _print()
+
+        # copy layout data
+        if self.enable_layout_data:
+            layout_data = {}
+            for nn in line.element_names:
+                if nn in mad.elements:
+                    madel = mad.elements[nn]
+                    # offset represent the offset of the assembly with respect to mid-beam
+                    eldata = {}
+                    eldata["offset"] = [madel.mech_sep / 2 * self.bv, madel.v_pos]
+                    eldata["assembly_id"] = madel.assembly_id
+                    eldata["slot_id"] = madel.slot_id
+                    eldata["aperture"] = [
+                        madel.apertype,
+                        list(madel.aperture),
+                        list(madel.aper_tol),
+                    ]
+                    layout_data[nn] = eldata
+
+            line.metadata["layout_data"] = layout_data
+
         return line
 
     def add_elements(
         self,
-        elements: List[ElementBuilder],
+        elements: List[Union[ElementBuilder]],
         line,
         buffer,
     ):
         out = {}  # tbc
         for el in elements:
-            xtel = el.add_to_line(line, buffer)
-            out[el.name] = xtel  # tbc
+            xt_element = el.add_to_line(line, buffer)
+            out[el.name] = xt_element  # tbc
         return out  # tbc
 
-    def convert_thin_element(self, xtrack_el, mad_el):
-        """add aperture and transformations to a thin element
+    @property
+    def math(self):
+        if issubclass(self.Builder, ElementBuilderWithExpr):
+            return self.line._var_management['fref']
+
+        import math
+        return math
+
+    def _assert_element_is_thin(self, mad_el):
+        if value_if_expr(mad_el.l) != 0:
+            if self.allow_thick:
+                raise NotImplementedError(
+                    f'Cannot load element {mad_el.name}, as thick elements of '
+                    f'type {"/".join(mad_el.get_type_hierarchy())} are not '
+                    f'yet supported.'
+                )
+            else:
+                raise ValueError(
+                    f'Element {mad_el.name} is thick, but importing thick '
+                    f'elements is disabled. Did you forget to set '
+                    f'`allow_thick=True`?'
+                )
+
+    def _make_drift_slice(self, mad_el, weight, name_pattern):
+        return self.Builder(
+            name_pattern.format(mad_el.name),
+            self.classes.Drift,
+            length=mad_el.l * weight,
+        )
+
+    def make_composite_element(
+            self,
+            xtrack_el,
+            mad_el,
+            custom_tilt=None,
+    ):
+        """Add aperture and transformations to a thin element:
         tilt, offset, aperture, offset, tilt, tilt, offset, kick, offset, tilt
+
+        Parameters
+        ----------
+        xtrack_el: list
+            List of xtrack elements to which the aperture and transformations
+            should be added.
+        mad_el: MadElement
+            The element for which the aperture and transformations should be
+            added.
+        custom_tilt: float, optional
+            If not None, the element will be additionally tilted by this
+            amount.
         """
-        align = Alignment(mad_el, self.enable_errors, self.classes, self.Builder)
-        # perm=self.permanent_alignement(mad_el) #to be implemented
-        elem_list = []
-        # elem_list.extend(perm.entry())
+        # TODO: Implement permanent alignment
+
+        align = Alignment(
+            mad_el, self.enable_align_errors, self.classes, self.Builder,
+            self.bv, custom_tilt)
+
+        aperture_seq = []
         if self.enable_apertures and mad_el.has_aperture():
-            aper = Aperture(mad_el, self.enable_errors, self)
-            elem_list.extend(aper.entry())
-            elem_list.extend(aper.aperture())
-            elem_list.extend(aper.exit())
-        elem_list.extend(align.entry())
-        elem_list.extend(xtrack_el)
-        elem_list.extend(align.exit())
-        # elem_list.extend(perm.exit())
+            if self.bv == -1:
+                raise NotImplementedError("Apertures for bv=-1 are not yet supported.")
+            aper = Aperture(mad_el, self.enable_align_errors, self)
+            aperture_seq = aper.aperture()
+
+        # using directly tilt and shift in the element
+        for xtee in xtrack_el:
+            if align.tilt or align.dx or align.dy:
+                xtee.rot_s_rad = align.tilt
+                if align.dx or align.dy:
+                    xtee.shift_x = align.dx
+                    xtee.shift_y = align.dy
+        align.tilt = 0
+        align.dx = 0
+        align.dy = 0
+
+        # Attach aperture to main element
+        if aperture_seq:
+            assert len(aperture_seq) <= 1, (
+                "Only one aperture per mad element is supported")
+            main_element=None
+            for ee in xtrack_el:
+                if ee.name== mad_el.name:
+                    main_element=ee
+                    break
+            assert main_element is not None
+            xtrack_el[0].name_associated_aperture = aperture_seq[0].name
+
+        elem_list = aperture_seq + xtrack_el
 
         return elem_list
 
+    def convert_quadrupole(self, mad_el):
+        if self.allow_thick:
+            if not mad_el.l:
+                raise ValueError(
+                    "Thick quadrupole with legth zero are not supported.")
+            return self._convert_quadrupole_thick(mad_el)
+        else:
+            raise NotImplementedError(
+                "Quadrupole are not supported in thin mode."
+            )
+
+    def _convert_quadrupole_thick(self, mad_el): # bv done
+
+        return self.make_composite_element(
+            [
+                self.Builder(
+                    mad_el.name,
+                    self.classes.Quadrupole,
+                    k1=self.bv * mad_el.k1,
+                    k1s=mad_el.k1s,
+                    length=mad_el.l,
+                ),
+            ],
+            mad_el,
+        )
+
+    def convert_rbend(self, mad_el): # bv done
+        return self._convert_bend(mad_el)
+
+    def convert_sbend(self, mad_el): # bv done
+        return self._convert_bend(mad_el)
+
+    def _convert_bend( # bv done
+        self,
+        mad_el,
+    ):
+
+        assert self.allow_thick, "Bends are not supported in thin mode."
+
+        l_curv = mad_el.l
+        h = mad_el.angle / l_curv
+
+        if mad_el.type == 'rbend' and self.sequence._madx.options.rbarc and value_if_expr(mad_el.angle):
+            R = 0.5 * mad_el.l / self.math.sin(0.5 * mad_el.angle) # l is on the straight line
+            l_curv = R * mad_el.angle
+            h = 1 / R
+
+        if not mad_el.k0:
+            k0 = h
+        else:
+            k0 = mad_el.k0
+
+        # Edge angles
+        if mad_el.type == 'sbend':
+            e1 = mad_el.e1
+            e2 = mad_el.e2
+        elif mad_el.type == 'rbend':
+            e1 = mad_el.e1 + mad_el.angle / 2
+            e2 = mad_el.e2 + mad_el.angle / 2
+        else:
+            raise NotImplementedError(
+                f'Unknown bend type {mad_el.type}.'
+            )
+
+        if self.bv == -1:
+            e1, e2 = e2, e1
+
+        # Convert bend core
+        num_multipole_kicks = 0
+        cls = self.classes.Bend
+        kwargs = {}
+        bend_core = self.Builder(
+            mad_el.name,
+            cls,
+            k0=k0,
+            h=h,
+            k1=self.bv * mad_el.k1,
+            length=l_curv,
+            edge_entry_angle=e1,
+            edge_exit_angle=e2,
+            edge_entry_angle_fdown=(k0 - h) * l_curv / 2,
+            edge_exit_angle_fdown=(k0 - h) * l_curv / 2,
+            edge_entry_fint=mad_el.fint,
+            edge_exit_fint=(
+                mad_el.fintx if value_if_expr(mad_el.fintx) >= 0 else mad_el.fint),
+            edge_entry_hgap=mad_el.hgap,
+            edge_exit_hgap=mad_el.hgap,
+            knl=[0, 0, mad_el.k2 * l_curv],
+            num_multipole_kicks=num_multipole_kicks,
+            **kwargs,
+        )
+
+        sequence = [bend_core]
+
+        return self.make_composite_element(sequence, mad_el)
+
+    def convert_sextupole(self, mad_el): # bv done
+        return self.make_composite_element(
+            [
+                self.Builder(
+                    mad_el.name,
+                    self.classes.Sextupole,
+                    k2=mad_el.k2,
+                    k2s=self.bv * mad_el.k2s,
+                    length=mad_el.l,
+                ),
+            ],
+            mad_el,
+        )
+
+    def convert_octupole(self, mad_el): # bv done
+        return self.make_composite_element(
+            [
+                self.Builder(
+                    mad_el.name,
+                    self.classes.Octupole,
+                    k3=self.bv*mad_el.k3,
+                    k3s=mad_el.k3s,
+                    length=mad_el.l,
+                ),
+            ],
+            mad_el,
+        )
+
+
     def convert_rectangle(self, mad_el):
         h, v = mad_el.aperture[:2]
         return [
@@ -697,12 +927,13 @@ class MadLoader:
         ]
 
     def convert_octagon(self, ee):
-        a0 = ee.aperture[0]
-        a1 = ee.aperture[1]
-        a2 = ee.aperture[2]
-        a3 = ee.aperture[3]
-        V1 = (a0, a0 * np.tan(a2))  # expression will fail
-        V2 = (a1 / np.tan(a3), a1)  # expression will fail
+        # MAD-X assumes X and Y symmetry, defines 2 points per quadrant
+        a0 = ee.aperture[0]  # half-width
+        a1 = ee.aperture[1]  # half-height
+        a2 = ee.aperture[2]  # angle between the lower point and the X axis
+        a3 = ee.aperture[3]  # angle between the other point and the X axis
+        V1 = (a0, a0 * self.math.tan(a2))
+        V2 = (a1 / self.math.tan(a3), a1)
         el = self.Builder(
             ee.name + "_aper",
             self.classes.LimitPolygon,
@@ -727,28 +958,46 @@ class MadLoader:
 
     def convert_marker(self, mad_elem):
         el = self.Builder(mad_elem.name, self.classes.Marker)
-        return self.convert_thin_element([el], mad_elem)
+        return self.make_composite_element([el], mad_elem)
 
     def convert_drift_like(self, mad_elem):
         el = self.Builder(mad_elem.name, self._drift, length=mad_elem.l)
-        return self.convert_thin_element([el], mad_elem)
+        return self.make_composite_element([el], mad_elem)
 
     convert_monitor = convert_drift_like
     convert_hmonitor = convert_drift_like
     convert_vmonitor = convert_drift_like
     convert_collimator = convert_drift_like
     convert_rcollimator = convert_drift_like
-    convert_collimator = convert_drift_like
     convert_elseparator = convert_drift_like
     convert_instrument = convert_drift_like
-    convert_solenoid = convert_drift_like
 
-    def convert_multipole(self, mad_elem):
+    def convert_solenoid(self, mad_elem): # bv done
+        if get_value(mad_elem.l) == 0:
+            _print(f'Warning: Thin solenoids are not yet implemented, '
+                   f'reverting to importing `{mad_elem.name}` as a drift.')
+            return self.convert_drift_like(mad_elem)
+
+        el = self.Builder(
+            mad_elem.name,
+            self.classes.Solenoid,
+            length=mad_elem.l,
+            ks=self.bv * mad_elem.ks,
+            ksi=self.bv * mad_elem.ksi,
+        )
+        return self.make_composite_element([el], mad_elem)
+
+    def convert_multipole(self, mad_elem): # bv done
+        self._assert_element_is_thin(mad_elem)
         # getting max length of knl and ksl
-        knl = mad_elem.knl
-        ksl = mad_elem.ksl
+        if self.bv == -1:
+            knl = [(-1)**ii * x for ii, x in enumerate(mad_elem.knl)]
+            ksl = [(-1)**(ii+1) * x for ii, x in enumerate(mad_elem.ksl)]
+        else:
+            knl = mad_elem.knl
+            ksl = mad_elem.ksl
         lmax = max(non_zero_len(knl), non_zero_len(ksl), 1)
-        if mad_elem.field_errors is not None and self.enable_errors:
+        if mad_elem.field_errors is not None and self.enable_field_errors:
             dkn = mad_elem.field_errors.dkn
             dks = mad_elem.field_errors.dks
             lmax = max(lmax, non_zero_len(dkn), non_zero_len(dks))
@@ -757,67 +1006,113 @@ class MadLoader:
         el = self.Builder(mad_elem.name, self.classes.Multipole, order=lmax - 1)
         el.knl = knl[:lmax]
         el.ksl = ksl[:lmax]
+
+        if hasattr(mad_elem, 'ksl') and mad_elem.ksl[0]:
+            raise NotImplementedError("Multipole with ksl[0] is not supported.")
+
+        if hasattr(el, 'hyl') and el.hyl:
+            raise NotImplementedError("Multipole with hyl is not supported.")
+
         if (
             mad_elem.angle
         ):  # testing for non-zero (cannot use !=0 as it creates an expression)
             el.hxl = mad_elem.angle
         else:
             el.hxl = mad_elem.knl[0]  # in madx angle=0 -> dipole
-            el.hyl = mad_elem.ksl[0]  # in madx angle=0 -> dipole
         el.length = mad_elem.lrad
-        return self.convert_thin_element([el], mad_elem)
+        return self.make_composite_element([el], mad_elem)
 
-    def convert_kicker(self, mad_elem):
-        hkick = [-mad_elem.hkick] if mad_elem.hkick else []
-        vkick = [mad_elem.vkick] if mad_elem.vkick else []
-        el = self.Builder(
-            mad_elem.name,
+    def convert_kicker(self, mad_el): # bv done
+        hkick = [-mad_el.hkick] if mad_el.hkick else []
+        vkick = [self.bv * mad_el.vkick] if mad_el.vkick else []
+        thin_kicker = self.Builder(
+            mad_el.name,
             self.classes.Multipole,
             knl=hkick,
             ksl=vkick,
-            length=mad_elem.lrad,
+            length=(mad_el.l or mad_el.lrad),
             hxl=0,
-            hyl=0,
         )
-        return self.convert_thin_element([el], mad_elem)
+
+        if value_if_expr(mad_el.l) != 0:
+            if not self.allow_thick:
+                self._assert_element_is_thin(mad_el)
+
+            sequence = [
+                self._make_drift_slice(mad_el, 0.5, "drift_{}..1"),
+                thin_kicker,
+                self._make_drift_slice(mad_el, 0.5, "drift_{}..2"),
+            ]
+        else:
+            sequence = [thin_kicker]
+
+        return self.make_composite_element(sequence, mad_el)
 
     convert_tkicker = convert_kicker
 
-    def convert_hkicker(self, mad_elem):
-        if mad_elem.hkick:
+    def convert_hkicker(self, mad_el): # bv done
+        if mad_el.hkick:
             raise ValueError(
                 "hkicker with hkick is not supported, please use kick instead")
-        hkick = [-mad_elem.kick] if mad_elem.kick else []
+
+        hkick = [-mad_el.kick] if mad_el.kick else []
         vkick = []
-        el = self.Builder(
-            mad_elem.name,
+        thin_hkicker = self.Builder(
+            mad_el.name,
             self.classes.Multipole,
             knl=hkick,
             ksl=vkick,
-            length=mad_elem.lrad,
+            length=(mad_el.l or mad_el.lrad),
             hxl=0,
-            hyl=0,
         )
-        return self.convert_thin_element([el], mad_elem)
 
-    def convert_vkicker(self, mad_elem):
-        if mad_elem.vkick:
+        if value_if_expr(mad_el.l) != 0:
+            if not self.allow_thick:
+                self._assert_element_is_thin(mad_el)
+
+            sequence = [
+                self._make_drift_slice(mad_el, 0.5, "drift_{}..1"),
+                thin_hkicker,
+                self._make_drift_slice(mad_el, 0.5, "drift_{}..2"),
+            ]
+        else:
+            sequence = [thin_hkicker]
+
+        return self.make_composite_element(sequence, mad_el)
+
+    def convert_vkicker(self, mad_el): # bv done
+        if mad_el.vkick:
             raise ValueError(
                 "vkicker with vkick is not supported, please use kick instead")
+
         hkick = []
-        vkick = [mad_elem.kick] if mad_elem.kick else []
-        el = self.Builder(
-            mad_elem.name,
+        vkick = [self.bv * mad_el.kick] if mad_el.kick else []
+        thin_vkicker = self.Builder(
+            mad_el.name,
             self.classes.Multipole,
             knl=hkick,
             ksl=vkick,
-            length=mad_elem.lrad,
+            length=(mad_el.l or mad_el.lrad),
             hxl=0,
-            hyl=0,
         )
-        return self.convert_thin_element([el], mad_elem)
+
+        if value_if_expr(mad_el.l) != 0:
+            if not self.allow_thick:
+                self._assert_element_is_thin(mad_el)
+
+            sequence = [
+                self._make_drift_slice(mad_el, 0.5, "drift_{}..1"),
+                thin_vkicker,
+                self._make_drift_slice(mad_el, 0.5, "drift_{}..2"),
+            ]
+        else:
+            sequence = [thin_vkicker]
+
+        return self.make_composite_element(sequence, mad_el)
 
     def convert_dipedge(self, mad_elem):
+        if self.bv == -1:
+            raise NotImplementedError("Dipole edges for bv=-1 are not yet supported.")
         # TODO LRAD
         el = self.Builder(
             mad_elem.name,
@@ -827,9 +1122,9 @@ class MadLoader:
             hgap=mad_elem.hgap,
             fint=mad_elem.fint,
         )
-        return self.convert_thin_element([el], mad_elem)
+        return self.make_composite_element([el], mad_elem)
 
-    def convert_rfcavity(self, ee):
+    def convert_rfcavity(self, ee): # bv done
         # TODO LRAD
         if ee.freq == 0 and ee.harmon:
             frequency = (
@@ -838,20 +1133,39 @@ class MadLoader:
         else:
             frequency = ee.freq * 1e6
         if (hasattr(self.sequence, 'beam')
-            and self.sequence.beam.particle == 'ion'):
+                and self.sequence.beam.particle == 'ion'):
             scale_voltage = 1./self.sequence.beam.charge
         else:
             scale_voltage = 1.
+        if self.bv == -1:
+            lag_deg = -ee.lag * 360 + 180
+        elif self.bv == 1:
+            lag_deg = ee.lag * 360
+        else:
+            raise ValueError(f"bv should be 1 or -1, not {self.bv}")
         el = self.Builder(
             ee.name,
             self.classes.Cavity,
             voltage=scale_voltage * ee.volt * 1e6,
             frequency=frequency,
-            lag=ee.lag * 360,
+            lag=lag_deg,
         )
-        return self.convert_thin_element([el], ee)
+
+        if value_if_expr(ee.l) != 0:
+            sequence = [
+                self._make_drift_slice(ee, 0.5, f"drift_{{}}..1"),
+                el,
+                self._make_drift_slice(ee, 0.5, f"drift_{{}}..2"),
+            ]
+        else:
+            sequence = [el]
+
+        return self.make_composite_element(sequence, ee)
 
     def convert_rfmultipole(self, ee):
+        if self.bv == -1:
+            raise NotImplementedError("RF multipole for bv=-1 are not yet supported.")
+        self._assert_element_is_thin(ee)
         # TODO LRAD
         if ee.harmon:
             raise NotImplementedError
@@ -868,9 +1182,12 @@ class MadLoader:
             pn=[v * 360 for v in ee.pnl],
             ps=[v * 360 for v in ee.psl],
         )
-        return self.convert_thin_element([el], ee)
+        return self.make_composite_element([el], ee)
 
     def convert_wire(self, ee):
+        if self.bv == -1:
+            raise NotImplementedError("Wire for bv=-1 are not yet supported.")
+        self._assert_element_is_thin(ee)
         if len(ee.L_phy) == 1:
             # the index [0] is present because in MAD-X multiple wires can
             # be defined within the same element
@@ -883,12 +1200,15 @@ class MadLoader:
                 xma=ee.xma[0],
                 yma=ee.yma[0],
             )
-            return self.convert_thin_element([el], ee)
+            return self.make_composite_element([el], ee)
         else:
             # TODO: add multiple elements for multiwire configuration
             raise ValueError("Multiwire configuration not supported")
 
     def convert_crabcavity(self, ee):
+        if self.bv == -1:
+            raise NotImplementedError("Crab cavity for bv=-1 are not yet supported.")
+        self._assert_element_is_thin(ee)
         # This has to be disabled, as it raises an error when l is assigned to an
         # expression:
         # for nn in ["l", "harmon", "lagf", "rv1", "rv2", "rph1", "rph2"]:
@@ -914,9 +1234,12 @@ class MadLoader:
                 pn=[ee.lag * 360 + 90],  # TODO: Changed sign to match sixtrack
                 # To be checked!!!!
             )
-        return self.convert_thin_element([el], ee)
+        return self.make_composite_element([el], ee)
 
     def convert_beambeam(self, ee):
+        if self.bv == -1:
+            raise NotImplementedError("BeamBeam for bv=-1 are not yet supported.")
+        self._assert_element_is_thin(ee)
         import xfields as xf
 
         if ee.slot_id == 6 or ee.slot_id == 60:
@@ -971,7 +1294,7 @@ class MadLoader:
                 d_px=0,
                 d_py=0,
             )
-        return self.convert_thin_element([el], ee)
+        return self.make_composite_element([el], ee)
 
     def convert_placeholder(self, ee):
         # assert not is_expr(ee.slot_id) can be done only after release MADX 5.09
@@ -1000,9 +1323,11 @@ class MadLoader:
             el = self.Builder(ee.name, self.classes.SCInterpolatedProfile)
         else:
             el = self.Builder(ee.name, self._drift, length=ee.l)
-        return self.convert_thin_element([el], ee)
+        return self.make_composite_element([el], ee)
 
     def convert_matrix(self, ee):
+        if self.bv == -1:
+            raise NotImplementedError("Matrix for bv=-1 are not yet supported.")
         length = ee.l
         m0 = np.zeros(6, dtype=object)
         for m0_i in range(6):
@@ -1018,39 +1343,55 @@ class MadLoader:
         el = self.Builder(
             ee.name, self.classes.FirstOrderTaylorMap, length=length, m0=m0, m1=m1
         )
-        return self.convert_thin_element([el], ee)
+        return self.make_composite_element([el], ee)
 
     def convert_srotation(self, ee):
+        if self.bv == -1:
+            raise NotImplementedError("SRotation for bv=-1 are not yet supported.")
         angle = ee.angle*180/np.pi
         el = self.Builder(
             ee.name, self.classes.SRotation, angle=angle
         )
-        return self.convert_thin_element([el], ee)
+        return self.make_composite_element([el], ee)
 
     def convert_xrotation(self, ee):
+        if self.bv == -1:
+            raise NotImplementedError("XRotation for bv=-1 are not yet supported.")
         angle = ee.angle*180/np.pi
         el = self.Builder(
             ee.name, self.classes.XRotation, angle=angle
         )
-        return self.convert_thin_element([el], ee)
+        return self.make_composite_element([el], ee)
 
     def convert_yrotation(self, ee):
+        if self.bv == -1:
+            raise NotImplementedError("YRotation for bv=-1 are not yet supported.")
         angle = ee.angle*180/np.pi
         el = self.Builder(
             ee.name, self.classes.YRotation, angle=angle
         )
-        return self.convert_thin_element([el], ee)
+        return self.make_composite_element([el], ee)
 
     def convert_translation(self, ee):
+        if self.bv == -1:
+            raise NotImplementedError("Translation for bv=-1 are not yet supported.")
         el_transverse = self.Builder(
             ee.name, self.classes.XYShift, dx=ee.dx, dy=ee.dy
         )
-        dzeta = ee.ds*self.sequence.beam.beta
-        el_longitudinal = self.Builder(
-            ee.name, self.classes.ZetaShift, dzeta=dzeta
-        )
+        if ee.ds:
+            raise NotImplementedError # Need to implement ShiftS element
         ee.dx = 0
         ee.dy = 0
         ee.ds = 0
-        return self.convert_thin_element([el_transverse,el_longitudinal], ee)
+        return self.make_composite_element([el_transverse], ee)
 
+    def convert_nllens(self, mad_elem):
+        if self.bv == -1:
+            raise NotImplementedError("Non-linear lens for bv=-1 are not yet supported.")
+        el = self.Builder(
+            mad_elem.name,
+            self.classes.NonLinearLens,
+            knll=mad_elem.knll,
+            cnll=mad_elem.cnll,
+        )
+        return self.make_composite_element([el], mad_elem)
