@@ -9,7 +9,6 @@ import xtrack as xt
 from ..base_element import BeamElement
 from ..general import _pkg_root
 
-
 def _monitor_init(
     self,
     _context=None,
@@ -25,6 +24,29 @@ def _monitor_init(
     auto_to_numpy=True,
 ):
 
+    '''
+    Beam element logging the coordinates of the particles passing through it.
+
+    Parameters
+    ----------
+    start_at_turn: int
+        Turn at which the monitor starts logging the particles coordinates.
+    stop_at_turn: int
+        Turn at which the monitor stops logging the particles coordinates.
+    n_repetitions: int
+        Number of times the monitor repeats the logging of the particles.
+    repetition_period: int
+        Period in number of turns for the repetition of the logging of the particles.
+    num_particles: int
+        Number of particles to be logged.
+    particle_id_range: tuple of int
+        Range of particle ids to be logged.
+    auto_to_numpy: bool
+        If True, the data is automatically converted to numpy arrays when
+        accessed.
+
+    '''
+
     if _xobject is not None:
         self.xoinitialize(_xobject=_xobject)
     else:
@@ -79,6 +101,19 @@ def _monitor_init(
             for tt, nn in self._ParticlesClass.per_particle_vars:
                 getattr(self.data, nn)[:] = 0
 
+def monitor_from_dict(cls, dct, **kwargs):
+    xobj = cls._XoStruct(**dct, **kwargs)
+    return cls(_xobject=xobj)
+
+def auto_to_numpy(self):
+    return self.flag_auto_to_numpy != 0
+
+def set_auto_to_numpy(self, flag):
+    if flag:
+        self.flag_auto_to_numpy = 1
+    else:
+        self.flag_auto_to_numpy = 0
+
 class _FieldOfMonitor:
     def __init__(self, name):
         self.name = name
@@ -105,7 +140,8 @@ def _monitor_get_backtrack_element(
 
     return xt.Marker(_context=_context, _buffer=_buffer, _offset=_offset)
 
-def generate_monitor_class(ParticlesClass):
+
+class ParticlesMonitor(BeamElement):
 
     _xofields = {
         "start_at_turn": xo.Int64,
@@ -116,32 +152,33 @@ def generate_monitor_class(ParticlesClass):
         "n_records": xo.Int64,
         "n_repetitions": xo.Int64,
         "repetition_period": xo.Int64,
-        "data": ParticlesClass._XoStruct,
+        "flag_auto_to_numpy": xo.Int64,
+        "data": xt.Particles,
     }
 
     _extra_c_sources = [
         _pkg_root.joinpath("monitors/particles_monitor.h")
     ]
 
-    ParticlesMonitorClass = type(
-        "ParticlesMonitor",
-        (BeamElement,),
-        {"_ParticlesClass": ParticlesClass,
-        '_xofields': _xofields,
-        '_extra_c_sources': _extra_c_sources,
-        },
-    )
+    behaves_like_drift = True
+    has_backtrack = True
+    allow_loss_refinement = True
+    _ParticlesClass = xt.Particles
+
+
+ParticlesMonitor.__init__ = _monitor_init
+ParticlesMonitor.get_backtrack_element = _monitor_get_backtrack_element
+ParticlesMonitor.from_dict = classmethod(monitor_from_dict)
+
+ParticlesMonitor.auto_to_numpy = property(auto_to_numpy, set_auto_to_numpy)
+
+per_particle_vars = xt.Particles.per_particle_vars
+for tt, nn in per_particle_vars:
+    setattr(ParticlesMonitor, nn, _FieldOfMonitor(name=nn))
+
+for nn in ['pzeta', 'kin_px', 'kin_py', 'kin_ps', 'kin_xprime', 'kin_yprime']:
+    setattr(ParticlesMonitor, nn, _FieldOfMonitor(name=nn))
 
-    ParticlesMonitorClass.__init__ = _monitor_init
-    ParticlesMonitorClass.get_backtrack_element = _monitor_get_backtrack_element
-    ParticlesMonitorClass.behaves_like_drift = True
-    ParticlesMonitorClass.allow_backtrack = True
 
-    per_particle_vars = ParticlesClass.per_particle_vars
-    for tt, nn in per_particle_vars:
-        setattr(ParticlesMonitorClass, nn, _FieldOfMonitor(name=nn))
 
-    for nn in ['pzeta']:
-        setattr(ParticlesMonitorClass, nn, _FieldOfMonitor(name=nn))
 
-    return ParticlesMonitorClass
