@@ -4,7 +4,7 @@
 # ######################################### //
 
 import xobjects as xo
-import xpart as xp
+import xtrack as xt
 
 from ..base_element import BeamElement
 from ..general import _pkg_root, _print
@@ -18,14 +18,11 @@ class RandomUniform(BeamElement):
         '_dummy': xo.UInt8,  # TODO: a hack for allocating empty struct on OCL
     }
 
-    iscollective = True
+    allow_track = False
 
     _extra_c_sources = [
-        # The base (bitwise) rng is in xpart, as this is where the
-        # seeds are stored. This is needed to avoid circular imports
-        # in xpart.Particles
-        xp._pkg_root.joinpath('rng_src','base_rng.h'),
-        _pkg_root.joinpath('random','random_src','uniform.h')
+        _pkg_root.joinpath('particles', 'rng_src', 'base_rng.h'),
+        _pkg_root.joinpath('random', 'random_src', 'uniform.h')
     ]
 
     _per_particle_kernels = {
@@ -41,18 +38,19 @@ class RandomUniform(BeamElement):
     def _sample(self, *args, **kwargs):
         self.sample_uniform(*args, **kwargs)
 
-    def track(self, *args, **kwargs):
-        raise RuntimeError("Random generators have no valid track method.")
-
-    def generate(self, n_samples=1000, n_seeds=1000, particles=None):
+    def generate(self, n_samples=1000, n_seeds=None, particles=None):
         context = self._context
-        n_seeds = int(n_seeds)
-        if particles is None:
-            particles = xp.Particles(state=np.ones(n_seeds),
-                                     x=np.ones(n_seeds), _context=context)
-        elif n_seeds != len(particles._rng_s1):
+        if n_seeds is None:
+            n_seeds = 1000
+        elif particles is not None:
             _print("Warning: both 'particles' and 'n_seeds' are given, but "
                   + "are not compatible. Ignoring 'n_seeds'...")
+        n_seeds = int(n_seeds)
+        if particles is None:
+            particles = xt.Particles(state=np.ones(n_seeds),
+                                     x=np.ones(n_seeds), _context=context)
+        else:
+            particles.move(_context=context)
             n_seeds = len(particles._rng_s1)
         if not particles._has_valid_rng_state():
             particles._init_random_number_generator()
@@ -68,7 +66,7 @@ class RandomUniform(BeamElement):
 
         return np.reshape(samples[:n_samples_per_seed*n_seeds],
                     (-1, n_samples_per_seed)
-                ), particles
+                )
 
 
 class RandomExponential(RandomUniform):
@@ -76,7 +74,7 @@ class RandomExponential(RandomUniform):
         '_dummy': xo.UInt8,  # TODO: a hack for allocating an empty struct on OCL
     }
 
-    iscollective = True
+    allow_track = False
 
     _depends_on = [RandomUniform]
 
@@ -103,7 +101,7 @@ class RandomNormal(RandomUniform):
         '_dummy': xo.UInt8,  # TODO: a hack for allocating an empty struct on OCL
     }
 
-    iscollective = True
+    allow_track = False
 
     _depends_on = [RandomUniform]
 
@@ -134,7 +132,7 @@ class RandomRutherford(RandomUniform):
         'Newton_iterations': xo.Int8
     }
 
-    iscollective = True
+    allow_track = False
 
     _depends_on = [RandomUniform]
 
@@ -183,7 +181,7 @@ class RandomRutherford(RandomUniform):
             raise ValueError('Rutherford random generator is not currently supported on GPU.')
 
     def set_parameters(self, A, B, lower_val, upper_val):
-        self.compile_kernels(only_if_needed=True)
+        self.compile_kernels(particles_class=xt.Particles, only_if_needed=True)
         context = self._buffer.context
         context.kernels.set_rutherford(rng=self, A=A, B=B, lower_val=lower_val, upper_val=upper_val)
 
